//*****************************************************
//
// Exemplo3DWireframe.cpp
// Um programa OpenGL que abre uma janela GLUT, desenha
// o wireframe de um objeto representado por uma malha
// de polgonos, e permite fazer zoom in e zoom out
// pressionando os botes do mouse (projeo perspectiva).
// IMPORTANTE:	 necessrio incluir no projeto a
//				biblioteca bibutil.cpp
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro
// "OpenGL - Uma Abordagem Prtica e Objetiva"
//
//*****************************************************

#include "bibutil.h"

// Variveis para controles de navegao
GLfloat angle, fAspect;
GLfloat rotX, rotY, rotX_ini, rotY_ini;
GLfloat obsX, obsY, obsZ, obsX_ini, obsY_ini, obsZ_ini;
int x_ini,y_ini,bot;

// Apontador para o objeto
OBJ *objeto;

// Funo callback de redesenho da janela de visualizao

void desenhaChao()
    {
        glColor3f(0,0,1);
        glLineWidth(1);
        glBegin(GL_LINES);
        for(float z=-1000; z<=1000; z+=10)
        {
            glVertex3f(-1000,-0.1f,z);
            glVertex3f( 1000,-0.1f,z);
        }
        for(float x=-1000; x<=1000; x+=10)
        {
            glVertex3f(x,-0.1f,-1000);
            glVertex3f(x,-0.1f,1000);
        }
        glEnd();
        glLineWidth(1);
    }
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Desenha o wireframe do objeto 3D lido do arquivo com a cor corrente

	desenhaChao();

	glColor3f(1.0f, 1.0f, 0.0f);
	glLineWidth(3);
    DesenhaObjeto(objeto);

	// Executa os comandos OpenGL
	//glFlush();
	glutSwapBuffers();
}

// Funo usada para especificar a posio do observador virtual
void PosicionaObservador(void)
{
	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();
	// Posiciona e orienta o observador
	glTranslatef(-obsX,-obsY,-obsZ);
	glRotatef(rotX,1,0,0);
	glRotatef(rotY,0,1,0);
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
	gluPerspective(angle,fAspect,0.1,1200);

	PosicionaObservador();
}

// Funo callback chamada quando o tamanho da janela  alterado
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);

	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo callback chamada para gerenciar eventos de teclas normais (ESC)
void Teclado (unsigned char tecla, int x, int y)
{
	if(tecla==27) // ESC ?
	{
		// Libera memria e finaliza programa
		LiberaObjeto(objeto);
		exit(0);
	}

}

void TeclasEspeciais (int tecla, int x, int y)
{
	switch (tecla)
	{
        //Altera escala do objeto
        case GLUT_KEY_UP:   escalaObjeto(objeto, 0.9); //-10%
							break;
        case GLUT_KEY_DOWN: escalaObjeto(objeto, 1.1); //+10%
							break;
	}
	PosicionaObservador();
	glutPostRedisplay();
}

// Funo callback para eventos de botes do mouse
void GerenciaMouse(int button, int state, int x, int y)
{
	if(state==GLUT_DOWN)
	{
		// Salva os parmetros atuais
		x_ini = x;
		y_ini = y;
		obsX_ini = obsX;
		obsY_ini = obsY;
		obsZ_ini = obsZ;
		rotX_ini = rotX;
		rotY_ini = rotY;
		bot = button;
	}
	else bot = -1;
}

// Funo callback para eventos de movimento do mouse
#define SENS_ROT	5.0
#define SENS_OBS	15.0
#define SENS_TRANSL	20.0
void GerenciaMovim(int x, int y)
{
	// Boto esquerdo ?
	if(bot==GLUT_LEFT_BUTTON)
	{
		// Calcula diferenas
		int deltax = x_ini - x;
		int deltay = y_ini - y;
		// E modifica ngulos
		rotY = rotY_ini - deltax/SENS_ROT;
		rotX = rotX_ini - deltay/SENS_ROT;
	}
	// Boto direito ?
	else if(bot==GLUT_RIGHT_BUTTON)
	{
		// Calcula diferena
		int deltaz = y_ini - y;
		// E modifica distncia do observador
		obsZ = obsZ_ini + deltaz/SENS_OBS;
	}
	else if(bot==GLUT_MIDDLE_BUTTON)
	{
		// Calcula diferenas
		int deltax = x_ini - x;
		int deltay = y_ini - y;
		// E modifica posies
		obsX = obsX_ini + deltax/SENS_TRANSL;
		obsY = obsY_ini - deltay/SENS_TRANSL;
	}
	PosicionaObservador();
	glutPostRedisplay();
}


// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{
	char nomeArquivo[30];

	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	// Inicializa a varivel que especifica o ngulo da projeo
	// perspectiva
	angle=54;

	// Inicializa as variveis usadas para alterar a posio do
	// observador virtual
	rotX = 0;
	rotY = 0;
	obsX = obsY = 0;
	obsZ = 10;

	// L o nome do arquivo e chama a rotina de leitura
	printf("Digite o nome do arquivo que contem o modelo 3D: ");
	gets(nomeArquivo);

	// Carrega o objeto 3D
	cout << nomeArquivo << endl;
	objeto = CarregaObjeto(nomeArquivo,false);
	//escalaObjeto(objeto,30);
	cout << "ok\n";
	system("pause");
	// Seta o modo de desenho como wireframe
	SetaModoDesenho('w');	// 's' para slido
}

// Programa Principal
int main(int argc, char** argv)
{
    glutInit(&argc, argv);

	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_DOUBLE| GLUT_RGB);

	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(300,300);

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(800,600);

	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Desenho do wireframe de um objeto 3D");

	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);

	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback para tratamento das teclas normais
	glutKeyboardFunc (Teclado);

	// Registra a funo callback para tratamento das teclas especiais
	glutSpecialFunc (TeclasEspeciais);

	// Registra a funo callback para eventos de botes do mouse
	glutMouseFunc(GerenciaMouse);

	// Registra a funo callback para eventos de movimento do mouse
	glutMotionFunc(GerenciaMovim);

	// Chama a funo responsvel por fazer as inicializaes
	Inicializa();

	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();

	return 0;
}
